var lr = require('tiny-lr'),
    del = require('del'),
    server = lr(),
    gulp = require('gulp'),
    livereload = require('gulp-livereload'),
    uglify = require('gulp-uglify'),
    webserver = require('gulp-webserver'),
    opn = require('opn'),
    sass = require('gulp-sass'),
    rename = require("gulp-rename"),
    zip = require('gulp-zip'),
    copy = require("gulp-copy"),
    sftp = require('gulp-sftp'),
    fileinclude = require('gulp-file-include'),
    config = require('./config.json'),
    gulpSequence = require('gulp-sequence'),
    postcss = require('gulp-postcss'),
    px2rem = require('postcss-px2rem');


//重命名project.md 文件
gulp.task('rename', function () {
    return gulp.src("./Project.md")
        .pipe(rename("README.md"))
        .pipe(gulp.dest("./build"));
});

//运行项目前清空view和css老数据 gulp的watch对删除文件不反应
gulp.task('clean:beforeInit', function (cb) {
    return del([
        'component/**/css/*',
        'view'
    ]);
});

//打包前清空build
gulp.task('clean:build', function (cb) {
    return del([
        'build'
    ])
});

//开启本地 Web 服务器功能
gulp.task('webserver', function () {
    gulp.src('./')
        .pipe(webserver({
            host: config.localserver.host,
            port: config.localserver.port,
            livereload: true,
            directoryListing: false
        }));
});

//通过浏览器打开本地 Web服务器 路径
gulp.task('openbrowser', function () {
    //opn('http://' + config.localserver.host + ':' + config.localserver.port + '/view/index.html');
});

//进行SASS 代码
gulp.task('sass', function () {
    var processors = [px2rem({remUnit: 75})];
    gulp.src('component/**/sass/*.scss')
        .pipe(sass({outputStyle:'compressed'}).on('error', function (err) {
            console.log(err.message)
        }))
        .pipe(rename(function (path) {
            path.dirname = path.dirname.replace('sass', 'css')
        }))
        .pipe(postcss(processors))
        .pipe(gulp.dest('component'));

    gulp.src('lib/mbiscroll/sass/*.scss')
        .pipe(sass({outputStyle:'compressed'}).on('error', function (err) {
            console.log(err.message)
        }))
        .pipe(rename(function (path) {
            path.dirname = path.dirname.replace('sass', 'css')
        }))
        .pipe(postcss(processors))
        .pipe(gulp.dest('lib/mbiscroll/css'))
});

//压缩component中javascript 文件
gulp.task('minifyjs', function () {
    return gulp.src('component/**/js/*')
        // .pipe(uglify())
        .pipe(gulp.dest('build/component'))
});

//文件监控
gulp.task('watch', function () {
    server.listen(35729, function (err) {
        if (err) {
            return console.log(err);
        }
    });

    gulp.watch('component/**/sass/*.scss', ['sass'])

    gulp.watch(['layout/*.html', 'widget/*.html'], ['fileinclude'])

    gulp.watch(['component/**/sass/*.scss', 'layout/*.html', 'widget/*.html', 'component/**/images/*', 'component/**/js/*.js'], function (e) {
        server.changed({
            body: {
                files: [e.path]
            }
        });
    });
});

//将widget整合渲染到layout中 同步编译 非打包
gulp.task('fileinclude', function () {
    gulp.src('layout/*.html')
        .pipe(fileinclude({
            prefix: '@@'
        }))
        .pipe(gulp.dest('view'));
});

//将相关项目文件复制到build 文件夹下
gulp.task('buildfiles', function () {
    //根目录文件
    gulp.src('component/**/**/*.{php,png,jpg,ico,svg}')
        .pipe(gulp.dest('build/component'));

    gulp.src('lib/**/*')
        .pipe(gulp.dest('build/lib'));

    gulp.src('component/**/fonts/*')
        .pipe(gulp.dest('build/component'));

    gulp.src('layout/*.html')
        .pipe(fileinclude({
            prefix: '@@'
        }))
        .pipe(gulp.dest('build/view'));
});

//css加md5
gulp.task('css-md', function() {
    return gulp.src('component/**/css/*.css')
        .pipe(gulp.dest('build/component')) //- 输出文件本地
});

//默认任务
gulp.task('default', gulpSequence(
    'clean:beforeInit', ['fileinclude','sass','watch','webserver','openbrowser']
));


//项目完成提交任务
gulp.task('build', gulpSequence(
    'clean:build', 'buildfiles','rename', ['minifyjs', 'css-md']
));

//打包主体build 文件夹并按照时间重命名
gulp.task('zip', function () {
    function checkTime(i) {
        if (i < 10) {
            i = "0" + i
        }
        return i
    }

    var d = new Date();
    var year = d.getFullYear();
    var month = checkTime(d.getMonth() + 1);
    var day = checkTime(d.getDate());
    var hour = checkTime(d.getHours());
    var minute = checkTime(d.getMinutes());

    return gulp.src('./build/**')
        .pipe(zip(config.project + '-' + year + month + day + hour + minute + '.zip'))
        .pipe(gulp.dest('./'));
});
